### Module containing helper functions to do misc stuff useful in various other files.

### Sadegh M, Fall 2016

from matplotlib import pyplot as plt
import numpy as np

def plotPredictions(mainData, predIndices=None, predValues=None, horiz=1):
    """
    Plots the bgl values in mainData as well as meal and bolus events. On top
    of these, predictions are also plotted, as well as true values computed
    based on horizon 'horiz' from the mainData.
    TAKES:
        mainData: NxD numpy matrix of data for a specific patient
        predIndices: prediction indices in mainData
        predValues: prediction values
        horiz: a value in units of resolution in mainData
    """

    N,D = mainData.shape

    plt.plot(mainData[:, 0])
    x = np.arange(N)[mainData[:, 1] > 0]
    plt.scatter(x, [0.5]*len(x), s=mainData[x, 1]*50, marker='s', c='b')

    x = np.arange(N)[mainData[:, 2] > 0]
    plt.scatter(x, [0.6]*len(x), s=mainData[x, 2]*70, c='y', edgecolor='r')

    if predIndices:
        plt.scatter(predIndices, mainData[predIndices, 0], s=70, marker='*', c='b')
        if predValues:
            plt.scatter([i+horiz for i in predIndices], predValues, s=50, c='r')

def callback_recordStates():
    """
    a callback function to record the LSTM state values when going over a sequence (for Keras models).
    """
    pass


def sigmoid(x):
    """ The sigmoid function """
    return 1 / (1 + np.exp(-x))


def areClose(x, y, rtol=1e-4, atol=1e-6):
    """ Returns True of x and y are close within some relative and absolute tolerances,
        Works exactly like np.isclose but is symmetric in x and y """
    return abs(x-y) <= (atol + rtol*(abs(x)+abs(y)))

def relDiff(x, y):
    """ Returns the relative symmetric difference of two values """
    return abs(x-y)/(abs(x)+abs(y))

def printPatientMeans(costVector, rList=None, nList = [39, 40, 40, 39, 38]):
    """ takes a cost vector containing 196 test point costs and prints the
        cost of each patient in the order in pid_list. When r is a list like nList,
        the function considers only the last r values of each patient.
        TAKES:
            nList: number of test points for each patient, in the order the are in costVector
            rList" the last r values of each patient, must have the same size as nList """
    i = 0
    exampsCovered = 0
    cTot = 0

    if not rList:
        rList = nList

    for n, r in zip(nList, rList):
        c = np.sum(costVector[i+n-r:i+n])
        print((c/r)**0.5)
        exampsCovered += r
        cTot += c
        i += n
    print("Overall RMSE: ", (cTot / exampsCovered)**0.5)
    print("cTot: ", cTot, " examps: ", exampsCovered)
